/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

/* osm_config.h needs to be before all other includes */
#include <opensm/osm_config.h>
#include <opensm/osm_version.h>
#include <opensm/osm_opensm.h>
#include <opensm/osm_log.h>

#include "cc_mgr_api.h"


//============================================================================//
/* globals */
static void (*smd_report)(void *plugin_data, osm_epi_event_id_t event_id,
                          void *event_data);


//============================================================================//
static void *construct(osm_opensm_t *p_osm)
{
    cl_list_item_t      *item;
    osm_epi_plugin_t    *plugin;
    int                 cnt = 0;

    smd_report = NULL;

    /* Find the SMD plugin so we can directly report to him */
    for (item  = cl_qlist_head(&p_osm->plugin_list);
         item != cl_qlist_end(&p_osm->plugin_list);
         item  = cl_qlist_next(item)) {

        /* Make sure we can't get stuck in infinite loop */
        if (cnt++ > 10)
            break;

        plugin = (osm_epi_plugin_t *)item;

        /* Only lookin for SMD */
        if (!plugin->plugin_name || strcmp(plugin->plugin_name, "smd"))
            continue;

        if (plugin->impl->report == NULL) {
            /* Programming error */
            continue;
        }

        smd_report = plugin->impl->report;

        break;
    }

    if (initCCMgr(p_osm,
            p_osm->subn.opt.event_plugin_options))
        fprintf(stdout, "CC Manager failed to load.\n");
    else
        fprintf(stdout, "CC Manager loaded successfully.\n");
    return (void *)p_osm;
}


//============================================================================//
static void destroy(void *data)
{
    smd_report = NULL;
    destroyCCMgr((osm_opensm_t*)data);
    fprintf(stdout, "CC Manager unloaded.\n");
}


//============================================================================//
static void report(void *data, osm_epi_event_id_t event_id, void *event_data)
{
    switch (event_id) {
    case OSM_EVENT_ID_SUBNET_UP:
        runCCMgr((osm_opensm_t*)data, event_id);
        break;
    case OSM_EVENT_ID_HEAVY_SWEEP_DONE:
        updateCCMgr((osm_opensm_t*)data, event_id);
        break;
    default:
        setOSMEventIdCCMgr((osm_opensm_t*)data, event_id);
        break;
    }
}


//============================================================================//
/**
 * Function that serves as interface between ccmgr and smd plug-in
 */
void do_smd_report(uint32_t msg_type, const int cnt, void *msg_data)
{
    osm_epi_event_id_t id;

    if (!smd_report)
        return;

    id = (osm_epi_event_id_t)msg_type;

    if (smd_report)
        (*smd_report)((void *)&cnt, id, (void *)msg_data);

    return;
}


//============================================================================//
/**
 * Define the object symbol for loading
 */
#if OSM_EVENT_PLUGIN_INTERFACE_VER != 2
    #error OpenSM plugin interface version missmatch
#endif


osm_event_plugin_t osm_event_plugin = {
    OSM_VERSION,
    construct,
    destroy,
    report
};
